

//******************************************************************//
//																	//
//				(C) 2017 R-MIK Sawomir Skrzyski					//
//				slawomir.skrzynski@ep.com.pl						//
//																	//
//	Programu nie mona rozpowszechnia bez pisemnej zgody autora!	//
//																	//
//******************************************************************//


#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#include "onewire.h"


#if defined( ONEWIRE_UART )
  #include "onewireUART.c"
#elif defined( ONEWIRE__DS2480 )
  #include "onewire2480.c"
#elif defined( ONEWIRE__DS2482 )
  #include "onewire2482.c"
#else
  #include "onewirePIO.c"
#endif


//----------------------------------------------------------------------//
/*
 * write one byte
 */
void onewireWriteByte(uint8_t dat){
	uint8_t i=8;


	onewireStrongPullup( FALSE );		// Wycz silne podciganie


  #ifdef ONEWIRE_UART
	while(i--)
		{
		onewireWriteBit(dat&1);
		dat >>= 1;
		}
  #else		//===== Tryb PIO =====//
	#if ONEWIRE_IRQ == 1
	  uint8_t cSREG;
	  cSREG = SREG; cli();
	#endif

	while(i--){
		onewireWriteBit(dat&1);
		dat >>= 1;
	}

	#if ONEWIRE_IRQ == 1
	  SREG = cSREG;
	#endif
  #endif
}


//----------------------------------------------------------------------//
/*
 * read one byte
 */
uint8_t onewireReadByte(void){
 	uint8_t dsByte=0;
	uint8_t i=8;


  #ifdef ONEWIRE_UART
	while(i--)
		{
		dsByte >>= 1;
		dsByte |= onewireReadBit() << 7;
		}
	return( dsByte );
  #else		//===== Tryb PIO =====//
	dsByte = 0;
	#if ONEWIRE_IRQ == 1
	  uint8_t cSREG;
	  cSREG = SREG; cli();
	#endif

	while(i--){
		dsByte >>= 1;
		dsByte |= (onewireReadBit()<<7);
	}

	#if ONEWIRE_IRQ == 1
	  SREG = cSREG;
	#endif
	return dsByte;
  #endif
}


//----------------------------------------------------------------------//
//				Adresowanie												//
//----------------------------------------------------------------------//
void onewriteMatchRom( byte *buf )
{
byte i;


	if ( onewireOverdrive )
		{
		onewriteMatchRomOVD( buf );
		}
	else
		{
		onewireWriteByte( ONEWIRE_CMD_MATCHROM ); 	// Match ROM
		for(i=0; i<8; i++)
			{
			onewireWriteByte( *buf++ );
			}
		}
}


//----------------------------------------------------------------------//
//TODO:
//Po przejciu do overdrive zamiast MATCHROM uywane jest resume.
// Wszystko ok dopuki komunikujemy sie cigle z tym samym ukadem, dlatego
// naley zapamita adres ukady i jeli jest inny ni ostatnio uy
// MATCHROM a nie RESUME.
//----------------------------------------------------------------------//
void onewriteMatchRomOVD( byte *buf )
{
byte static ov_match_mem[8] = { 0 };
byte i, compare=TRUE;


	#ifndef ONEWIRE_OVERDRIVE_OFF
	for(i=0; i<8; i++)								// Czy adresy zgodne ?
		{
		if ( *(buf+i) != ov_match_mem[i] )			// Adresy rne, wic
			{
			compare = FALSE;
			break;
			}
		}


	if ( onewireOverdrive & compare )				// Zwikszona prdko i adres zgodny wic resume
		{
		onewireWriteByte( ONEWIRE_CMD_RESUME );	// Resume
		}
	else
		{
		onewireWriteByte( ONEWIRE_OVD_MATCHROM ); 	// Match ROM
		onewriteOverdriveOnOff( TRUE );				// Zmieniamy prdko
		for(i=0; i<8; i++)
			{
			ov_match_mem[i] = *buf;					// Zapamitaj adres do porwnania
			onewireWriteByte( *buf++ );
			}
		}
	#else
	  onewriteMatchRom( buf );
	#endif
}


//----------------------------------------------------------------------//
byte OWtestOverdrive()
{
byte result=FALSE;


	//----- Test czy cho jedno urzdzenie obsuguje tryb overdrive
	onewireResetSTD();
	onewireWriteByte( ONEWIRE_CMD_SKIPROM );
	result = onewireResetOVD();

	onewireResetSTD();
	return( result );
}


//----------------------------------------------------------------------//
uint8_t onewireResetSTD()
{
	onewriteOverdriveOnOff( FALSE );
	onewireReset();
}


//----------------------------------------------------------------------//
uint8_t onewireResetOVD()
{
	onewriteOverdriveOnOff( TRUE );
	onewireReset();
}


//----------------------------------------------------------------------//
uint8_t onewireCrc8update(uint8_t crc, uint8_t data) {
//	uint8_t i;

    return( _crc_ibutton_update( crc, data) );

/*
	crc = crc ^ data;

	for (i = 0; i < 8; ++i) {
		if (crc & 0x01) {
			crc = (crc >> 1) ^ 0x8c;
		} else {
			crc >>= 1;
		}
	}
	return( crc );
*/
}


void onewriteOverdriveOnOff( byte ovd )
{
	onewireOverdrive = ovd;


	#ifndef DS2482_ver		// Jeli biblioteka istnieje
		byte DS2482_ver;
		if( DS2482_ver )	// Ukad wykryty
			{
			DS2482_OverdriveOnOff( ovd );
			}
	#endif
}


//----------------------------------------------------------------------//
//				Silne podcignie										//
//----------------------------------------------------------------------//
void onewireStrongPullup( byte pullup )
{
#warning "dla DS18B20, 2431 onewireStrongPullup przy pomiarze/programowaniu zastapic przez onewirePulse(5)"
	if ( !onewirePullup ) return;


	ONEWIREUP_PORT |= (1<<ONEWIREUP_DQ);		// Port stan wysoki (podcigaie silne lub sabe)

	if ( pullup )
		{
		ONEWIREUP_DDR |= (1<<ONEWIREUP_DQ);		// Wyjcie (silne podciganie)
		#if defined( ONEWIRE_UART ) && defined( ONEWIREUP_RXD )
		  OW_UCSRB &= ~(1<<OW_RXEN);			// Disable receiver
		#endif
		}
	else
		{
		ONEWIREUP_DDR	&= ~(1<<ONEWIREUP_DQ);	// Wejcie (sabe podciganie)
		#if defined( ONEWIRE_UART ) && defined( ONEWIREUP_RXD )
		  OW_UCSRB |= (1<<OW_RXEN);				// Enable receiver
		#endif
		}
}


//----------------------------------------------------------------------//
//						Generuje impuls programujcy					//
//----------------------------------------------------------------------//
byte onewirePulse( byte vpp, word czas )
{
	byte strongMem;


#warning "Vpp"
	if ( vpp == 12 )
		{
//todo: programowanie
// Wczy przetwornic
// odczeka czas (sprawdzi na oscyloskopie)
//todo: kontrola Vpp:
// jeli rev1701 - zmierzy napicie przez ADC
// jeli DS2480 (bajt wersji) to sprawdzi dodatkowo czy (po komendzie reset ???) stwierdzono Vpp
		onewireVppErr = OW_ERRVPP_FAULT;
		return( false );
// jeli nie DS2480 to wysterowa tranzystor
// sprawdzi napicie na magistrali, jeli nie ma vpp to brak/uszkodzony tranzystor/DS2480
		onewireVppErr = OW_ERRVPP_DRIVER;
		return( false );
		_dealy_ms( czas );
// wycza tranzystor Vpp
// wycz przetwornic
		onewireVppErr = 0;
		return( true );
		}
	else if ( vpp == 5 )
		{
		strongMem = onewirePullup;		// Zapamitaj ustawienie
		onewirePullup = true;
		onewireStrongPullup( true );	// Wcz silne podciganie
		onewirePullup = strongMem;		// Przywr ustawienie
		onewireVppErr = 0;
		return( true );
		}


	onewireVppErr = OW_ERRVPP_VOLTAGE;
	return( false );
}


//----------------------------------------------------------------------//
