

//******************************************************************//
//																	//
//				(C) 2017 R-MIK Sawomir Skrzyski					//
//				slawomir.skrzynski@ep.com.pl						//
//																	//
//	Programu nie mona rozpowszechnia bez pisemnej zgody autora!	//
//																	//
//******************************************************************//


#include <rmik/onewire_DS2431.h>;


//=======================================================================//
//							DS2431
//----------------------------------------------------------------------//
//#define	DS2431_EMU		// Emuluje eeprom ukadu w eeprom cpu


#ifdef ONEWIRE_UART
  #warning "Tryb overdriwe niemozliwy"
#endif


//----------------------------------------------------------------------//
#ifdef DS2431_EMU
  static byte ds2431_sp[8];
  byte ds2431_ee[144] EEMEM =
  	{
	0, 0, 0, 0, 0, 0, 0, 0,			// Page 0
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,			// Page 1
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,			// Page 2
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,			// Page 3
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0x55, 0, 0		// Cfg
	};
#endif


//----------------------------------------------------------------------//
//							Odczyt eeprom
//----------------------------------------------------------------------//
byte ds2431_read( byte skip, byte *match, byte *dst, word TA, byte len )
{
byte a, result=FALSE;


	if ( len > 0x90 ) len = 0x90;


	#if ONEWIRE_IRQ == 2
	  uint8_t cSREG;
	  cSREG = SREG; cli();
	#endif


	if ( onewireReset() ) 										//reset
		{
		//----- EEprom to  Ram -----//
		#ifdef ONEWIRE_OVERDRIVE
#warning "zaimplementowac w DS2431"
//to samo take dla
/*
	#ifdef OW_SEARCH_ROM_NO
	  if ( onewireLoopBack ) ???	// Jeli ptla lokalna nie pozwalaj na overdrive
	  								// podmie rozkazy ONEWIRE_OVD_xxx na ONEWIRE_CMD_
	#endif
*/
		if ( skip ){ onewireWriteByte( ONEWIRE_OVD_SKIPROM ); }//skip ROM overdrive
		  else{ onewriteMatchRomOVD( (byte*)match ); }			//match ROM overdrive
		#else
		  if ( skip ){ onewireWriteByte( ONEWIRE_CMD_SKIPROM ); }//skip ROM
		  else{ onewriteMatchRom( (byte*)match ); }				//match ROM
		#endif

		onewireWriteByte( ONEWIRE_2431_readMEMORY );

		onewireWriteByte( TA );					// Ofset
		onewireWriteByte( TA >> 8 );			// Adres

		_delay_us( OWDELAYPULLUP );				// Czas na naadowanie magistrali

		for(a=0; a<len; a++)
			{
			#ifndef DS2431_EMU
			  *dst++ = onewireReadByte();
			#else
			  *dst++ = eeprom_read_byte( &ds2431_ee[TA+a] );
			#endif
			}

		result = TRUE;
		}


	#if ONEWIRE_IRQ == 2
	  SREG = cSREG;
	#endif


	//----- RESET -----//
//	onewriteOverdriveOnOff( FALSE );	//todo: mona nie wycza ale trzeba to zrobi na pocztku obsugi ukadw bez tego trybu
	onewireReset();
	return( result );
}


//----------------------------------------------------------------------//
//			Zapisuje 8 bajtw z eeprom
//----------------------------------------------------------------------//
byte ds2431_wr( byte skip, byte *match, byte *src, word TA )
{
#define	ES	8
static byte a, result;
static byte ta1r, ta2r, esr, Status;
static word crc16, RDcrc;
static byte tabr[8];
	//todo: static test

//todo: !!!!
// szybka komunikacja (w planach) na razie tylko w trybie pio


	TA &= 0xFFF8;				// W adresie bity 0..3 = 0


	#if ONEWIRE_IRQ == 2
	  uint8_t cSREG;
	  cSREG = SREG; cli();
	#endif


	TA &= 0xFFF8;				// bity 0..3 = 0


	if ( onewireReset() ) 										//reset
		{
		//----- Ram to Scratchpad -----//
		#ifdef ONEWIRE_OVERDRIVE
		  if ( skip ){ onewireWriteByte( ONEWIRE_OVD_SKIPROM ); }//skip ROM overdrive
		  else{ onewriteMatchRomOVD( (byte*)match ); }			//match ROM overdrive
		  onewriteOverdriveOnOff( TRUE );
		#else
		  if ( skip ){ onewireWriteByte( ONEWIRE_CMD_SKIPROM ); }//skip ROM
		  else{ onewriteMatchRom( (byte*)match ); }				//match ROM
		#endif

		onewireWriteByte( ONEWIRE_2431_writeSCRATCHPAD );

		crc16=0xFFFF;
		onewireWriteByte( TA );					// Ofset
		crc16 = _crc16_update( crc16, TA );
		onewireWriteByte( TA >> 8 );			// Adres
		crc16 = _crc16_update( crc16, TA>>8 );
		for(a=0; a<ES; a++)
			{
			crc16 = _crc16_update( crc16, *src );
			#ifdef DS2431_EMU
			  ds2431_sp[a] = *src;
			#endif
			onewireWriteByte( *src );			// Zapisz dane
			*src++;
			}
		_delay_us( OWDELAYPULLUP );				// Czas na naadowanie magistrali

		RDcrc = onewireReadByte();				// Czytaj zanegowane CRC
		RDcrc |= onewireReadByte() << 8;
		RDcrc ^= 0xFFFF;


		//----- Scratchpad to Ram -----//
		onewireReset();
		#ifdef ONEWIRE_OVERDRIVE
		  if ( skip ){ onewireWriteByte( ONEWIRE_OVD_SKIPROM ); }//skip ROM overdrive
		  else{ onewriteMatchRomOVD( (byte*)match ); }			//match ROM overdrive
		  onewriteOverdriveOnOff( TRUE );
		#else
		  if ( skip ){ onewireWriteByte( ONEWIRE_CMD_SKIPROM ); }//skip ROM
		  else{ onewriteMatchRom( (byte*)match ); }				//match ROM
		#endif

		onewireWriteByte( ONEWIRE_2431_readSCRATCHPAD );
		_delay_us( OWDELAYPULLUP );				// Czas na naadowanie magistrali

		crc16=0xFFFF;
		ta1r = onewireReadByte();				// Ofset
		crc16 = _crc16_update( crc16, tabr[ a ] );
		ta2r = onewireReadByte();				// Adres
		crc16 = _crc16_update( crc16, tabr[ a ] );
		esr = onewireReadByte();				// Read E/S, ending offset = 111b, AA, PF = 0
		crc16 = _crc16_update( crc16, tabr[ a ] );
//todo: if ( esr & 0x20 ) bd zapisu pamici podrcznej
		for(a=0; a<ES; a++)
			{
			crc16 = _crc16_update( crc16, (tabr[a]=onewireReadByte()) );
			}
		RDcrc = onewireReadByte();				// Czytaj zanegowane CRC
		RDcrc |= onewireReadByte() << 8;
		RDcrc ^= 0xFFFF;

		//----- Zapis EEprom -----//
		#ifndef DS2431_EMU
		  onewireReset();
		#ifdef ONEWIRE_OVERDRIVE
		  if ( skip ){ onewireWriteByte( ONEWIRE_OVD_SKIPROM ); }//skip ROM overdrive
		  else{ onewriteMatchRomOVD( (byte*)match ); }			//match ROM overdrive
		  onewriteOverdriveOnOff( TRUE );
		#else
		  if ( skip ){ onewireWriteByte( ONEWIRE_CMD_SKIPROM ); }//skip ROM
		  else{ onewriteMatchRom( (byte*)match ); }				//match ROM
		#endif

		  onewireWriteByte( ONEWIRE_2431_copySCRATCHPAD );

		  onewireWriteByte( ta1r );				// Ofset
		  onewireWriteByte( ta2r );				// Adres
		  onewireWriteByte( esr );				// Read E/S, ending offset = 111b, AA, PF = 0
		  onewireStrongPullup( TRUE );
		  _delay_ms(10);
		  onewireStrongPullup( FALSE );

		  if ( (Status=onewireReadByte()) == 0xAA ) result = TRUE;
		#else
		  ds2431wr_EMU( ta1r | (ta2r << 8) );
		  Status = 0xAA; result = TRUE;
		#endif

		//----- ES to Ram -----//
		onewireReset();
		#ifdef ONEWIRE_OVERDRIVE
		  if ( skip ){ onewireWriteByte( ONEWIRE_OVD_SKIPROM ); }//skip ROM overdrive
		  else{ onewriteMatchRomOVD( (byte*)match ); }			//match ROM overdrive
		  onewriteOverdriveOnOff( TRUE);
		#else
		  if ( skip ){ onewireWriteByte( ONEWIRE_CMD_SKIPROM ); }//skip ROM
		  else{ onewriteMatchRom( (byte*)match ); }				//match ROM
		#endif

		onewireWriteByte( ONEWIRE_2431_readSCRATCHPAD );
		_delay_us( OWDELAYPULLUP );				// Czas na naadowanie magistrali

		ta1r = onewireReadByte();				// Ofset
		ta2r = onewireReadByte();				// Adres
		esr = onewireReadByte();				// Read E/S, ending offset = 111b, AA, PF = 0
		}


	//----- EXIT -----//
	#if ONEWIRE_IRQ == 2
	  SREG = cSREG;
	#endif
	//----- RESET -----//
//	onewriteOverdriveOnOff( FALSE );	//todo: mona nie wycza ale trzeba to zrobi na pocztku obsugi ukadw bez tego trybu
	onewireReset();
	return( result );
}


//----------------------------------------------------------------------//
//			Zapisuje eeprom / eprom/ (rom przed ustawieniem blokady)	//
//----------------------------------------------------------------------//
byte ds2431_write( byte skip, byte *match, byte *src, word Ta )
{
	if ( Ta >= 0x80 ) return( FALSE );
	if ( Ta & 7 ) return( FALSE );
	return( ds2431_wr( skip, match, src,  Ta ) );
}


//----------------------------------------------------------------------//
//			Ustawia obszar jako pami eprom							//
//----------------------------------------------------------------------//
byte ds2431_user( byte skip, byte *match, byte userL, byte userH )
{
	if ( ds2431_protect( skip, match, userL, 0x86 ) &&
		 ds2431_protect( skip, match, userH, 0x87 ) ) return( TRUE);


	return( FALSE );
}


//----------------------------------------------------------------------//
//			Ustawia obszar jako pami eprom							//
//----------------------------------------------------------------------//
byte ds2431_protectEPROM( byte skip, byte *match, byte protect, word TA )
{
	// "protect" ignorujemy
	if ( TA > 0x83 ) return( FALSE );
	return( ds2431_protect( skip, match, 0xAA, TA ) );
}


//----------------------------------------------------------------------//
//			Ustawia obszar jako pami ROM								//
//----------------------------------------------------------------------//
byte ds2431_protectROM( byte skip, byte *match, byte protect, word TA )
{
	// "protect" ignorujemy
	if ( TA > 0x83 ) return( FALSE );
	return( ds2431_protect( skip, match, 0x55, TA ) );
}


//----------------------------------------------------------------------//
//			Blokuje zmian konfiguracji i bitw USER					//
//----------------------------------------------------------------------//
byte ds2431_protectGLOBAL( byte skip, byte *match, byte protect, word TA )
{
	// "protect" naley ustawi na $55
	if ( TA != 0x84 ) return( FALSE );
	return( ds2431_protect( skip, match, protect, TA ) );
}


//----------------------------------------------------------------------//
//			Ustawia bajty $80..$87 z wyczeniem $85					//
//----------------------------------------------------------------------//
byte ds2431_protect( byte skip, byte *match, byte protect, word TA )
{
static byte FF[8]={0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};
static byte x, adr, df2431cfg[8];	//todo: static test
byte crc=0, crc2=0;


	#ifdef ONEWIRE_UART
	  #warning "Zapis eeprom DS2431 w trybie UART niemozliwy"
	#endif


	if ( TA<0x80 || TA>0x87) return( FALSE );		// Bd jeli zy zakres adresw


	crc = crc2 = 0;
	ds2431_read( skip, match, df2431cfg, 0x80, 8 );	// Odczyt eeprom
	for(x=0; x<8; x++) crc += df2431cfg[x];
	ds2431_read( skip, match, df2431cfg, 0x80, 8 );	// Ponowny odczyt
	for(x=0; x<8; x++) crc2 += df2431cfg[x];
	if( crc != crc2 ) return( FALSE );				// Weryfinacja odczytw

	for(x=0; x<6; x++) df2431cfg[x] = 0;			// Zerowanie konfigu

	df2431cfg[TA & 0x07] = protect;					// Zabezpiecz
	adr = (TA & 0x07) << 5;							// Adres strony
	if ( protect==0xAA && TA>=0x80 && TA <=0x83 )	// Jeli eeprom mode stron 0..3
		{
		for(x=0; x<32/8; x++) ds2431_wr( skip, match, FF, adr+(x<<3) ); // Wypenij stron 0xFF
		}
nop();
	return( ds2431_wr( skip, match, df2431cfg,  TA ) );
nop();
}


//----------------------------------------------------------------------//
//			Emuluje pami DS2431 w RAM CPU								//
//----------------------------------------------------------------------//
#ifdef DS2431_EMU
void ds2431wr_EMU( word _TA )
{
static	  byte x, sreg, dana, tryb, ds2431_8x[16], ds2431_Gprotrct=FALSE;
	//todo test static


	// Kopia bitw konfiguracyjnych
	for(x=0; x<16; x++) ds2431_8x[x] = eeprom_read_byte( &ds2431_ee[0x80+x] );
	tryb=-1;

	if ( (ds2431_8x[4]==0xAA) || (ds2431_8x[4]==0x55) ) ds2431_Gprotrct = TRUE;

	switch( (_TA & 0xFFE0) )
		{
		case( 0x00 ):				// Strona 0
			if ( ds2431_8x[0] == 0xAA ) tryb = 'e';	// tryb EPROM
			if ( ds2431_8x[0] == 0x55 ) tryb = 'p';	// tryb ROM
			if ( ds2431_8x[0] == 0xAA && ds2431_Gprotrct ) tryb = 'p';	// tryb ROM bo (EPROM ale zabezpieczenie w $84)
			break;
		case( 0x20 ):				// Strona 1
			if ( ds2431_8x[1] == 0xAA ) tryb = 'e';	// tryb EPROM
			if ( ds2431_8x[1] == 0x55 ) tryb = 'p';	// tryb ROM
			if ( ds2431_8x[1] == 0xAA && ds2431_Gprotrct ) tryb = 'p';	// tryb ROM bo (EPROM ale zabezpieczenie w $84)
			break;
		case( 0x40 ):				// Strona 2
			if ( ds2431_8x[2] == 0xAA ) tryb = 'e';	// tryb EPROM
			if ( ds2431_8x[2] == 0x55 ) tryb = 'p';	// tryb ROM
			if ( ds2431_8x[2] == 0xAA && ds2431_Gprotrct ) tryb = 'p';	// tryb ROM bo (EPROM ale zabezpieczenie w $84)
			break;
		case( 0x60 ):				// Strona 3
			if ( ds2431_8x[3] == 0xAA ) tryb = 'e';	// tryb EPROM
			if ( ds2431_8x[3] == 0x55 ) tryb = 'p';	// tryb ROM
			if ( ds2431_8x[3] == 0xAA && ds2431_Gprotrct ) tryb = 'p';	// tryb ROM bo (EPROM ale zabezpieczenie w $84)
			break;
		case( 0x80 ):				// Config
			tryb = 'c';
			break;
		}
	//----- Zapis EEPROM obszaru uytkownika -----//
	if ( tryb != 'c' )
		{
		for(x=0; x<8; x++)
		  	{
			dana = ds2431_sp[x];
			if ( tryb == 'e' ) dana &= eeprom_read_byte( &ds2431_ee[ (_TA & 0xFFF8)+x ] );	// Tryb EPROM; dana = dana & eeprom
			if ( tryb == 'p' ) dana = eeprom_read_byte( &ds2431_ee[ (_TA & 0xFFF8)+x ] );	// Tryb ROM; dana = eeprom
			sreg = SREG;
		  	eeprom_write_byte( &ds2431_ee[ (_TA & 0xFFF8)+x ], dana );			// Adres z bitami 0..3 = 0 + x
			SREG = sreg;
	  		}
nop();
		}
	//----- Zapis EEPROM obszaru konfiguracji -----//
	if ( tryb == 'c' )			// if adres == config
		{
		for(x=0; x<8; x++)
		  	{
			dana = ds2431_sp[x];
			switch ( x )			// analizujemy bity 0..3 config'u
				{
				case( 0 ):			// $80 - Zabezpieczenie strony 0
					if ( ds2431_8x[0]==0xAA || ds2431_8x[0]==0x55 || ds2431_Gprotrct )	// Jeli protect ON
						{																// nie modyfikuj bajtu
						dana = ds2431_8x[0];
						}
					break;
				case( 1 ):			// $81 - Zabezpieczenie strony 1
					if ( ds2431_8x[1]==0xAA || ds2431_8x[1]==0x55 || ds2431_Gprotrct )	// Jeli protect ON
						{																// nie modyfikuj bajtu
						dana = ds2431_8x[1];
						}
					break;
				case( 2 ):			// $82 - Zabezpieczenie strony 2
					if ( ds2431_8x[2]==0xAA || ds2431_8x[2]==0x55 || ds2431_Gprotrct)	// Jeli protect ON
						{																// nie modyfikuj bajtu
						dana = ds2431_8x[2];
						}
					break;
				case( 3 ):			// $83 - Zabezpieczenie strony 3
					if ( ds2431_8x[3]==0xAA || ds2431_8x[3]==0x55 || ds2431_Gprotrct)	// Jeli protect ON
						{																// nie modyfikuj bajtu
						dana = ds2431_8x[3];
						}
					break;
				case( 4 ):			// $84 - Zabezpieczenie globalne, ju nic nie da sie zmieni w konfiguracji
					if ( ds2431_Gprotrct )				// Jeli protect ON
						{								// nie modyfikuj bajtu
						dana = ds2431_8x[4];
						}
					else
						{
						if ( dana==0xAA || dana==0x55 )	// Jeli OFF a ustawienie na ON
							{
							ds2431_8x[5] = 0xAA;		// zabezpiecz dostpn do $86 i $87
							}
						}
					break;
				case( 5 ):			// $85				// Tylko do odczytu
					dana = ds2431_8x[5];
					break;
				case( 6 ):			// $86 (user L)
					if ( ds2431_8x[5] == 0xAA || ds2431_Gprotrct )	// Jeli protect ON
						{											// nie modyfikuj bajtu
						dana = ds2431_8x[6];
						}
					break;
				case( 7 ):			// $87 (user H)
					if ( ds2431_8x[5] == 0xAA || ds2431_Gprotrct )	// Jeli protect ON
						{							// nie modyfikuj bajtu
						dana = ds2431_8x[7];
						}
					break;
				}
			sreg = SREG;
			eeprom_write_byte( &ds2431_ee[ (_TA & 0xFFF8)+x ], dana );			// Adres z bitami 0..3 = 0 + x
			SREG = sreg;
			}
nop();
		}


nop();
}
#endif


//=======================================================================//
